import pandas as pd
from sqlalchemy import create_engine
from sqlalchemy.engine import URL
from dotenv import dotenv_values
from scipy.stats import ttest_ind, binomtest
from typing import NamedTuple, List
import argparse
from natsort import natsort_keygen


class Args(NamedTuple):
    """Command Line Arguments"""
    analysis_name:str
    env_file:str
    gene_col:str
    pathway_col:str
    pathway_gene_table:str
    reference_organism:str
    expected_abundance_table:str
    control_samples_table:str
    value_transformation:str
    sample_table:str
    sample_id_col:str


def get_args() -> Args:
    """Get command line arguments"""
    parser = argparse.ArgumentParser(description='NeuroProteomics Analysis Tool', formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument("analysis_name", help="Name/description of analysis", type=str, metavar="ANALYSIS_NAME")
    parser.add_argument("env_file", help="Path to .env file containing DB connection information", type=str, metavar=".ENV FILE", default=".env")
    parser.add_argument("-g","--gene_col", help="Name of column that contains short gene names (ex. Lipa for lysosomal acid lipase A)", type=str, metavar="GENE_COL", default="gene'")
    parser.add_argument("-p","--pathway_col", help="Name of column that contains pathway names", type=str, metavar="PATHWAY_COL", default='pathway')
    parser.add_argument("-t","--pathway_gene_table", help="Name of table in DB that contains genes info (gene name, pathway, etc)", type=str, metavar="PATHWAY_GENE_TABLE", default="pathway_genes")
    parser.add_argument("-r", "--reference_organism", help="Reference organism to use for analysis", type=str, metavar="REFERENCE_ORGANISM", default="Mus musculus", choices=["Mus musculus"])
    parser.add_argument("-e", "--expected_abundance_table", help="Table containing expected abundances", type=str, metavar="EXPECTED_ABUNDANCE", default="expected_abundances")
    parser.add_argument("-c", "--control_samples_table", help="Table containing control samples", type=str, metavar="CONTROL_SAMPLES", default="control_samples")
    parser.add_argument("-v", "--value_transformation", help="Type of transformation to apply before analysis", type=str, metavar="VALUE_TRANSFORMATION", default="log2", choices=["log2"])
    parser.add_argument("-s","--sample_table", help="Name of table in DB that contains samples", type=str, metavar="SAMPLE_TABLE", default='test_input')
    parser.add_argument("-i","--sample_id_col", help="Name of column in sample_table that contains sample IDs", type=str, metavar="SAMPLE_ID_COL", default='sample_id')





    args = parser.parse_args()
    return Args(analysis_name=args.analysis_name,
                env_file=args.env_file,
                gene_col=args.gene_col,
                pathway_col=args.pathway_col,
                pathway_gene_table=args.pathway_gene_table,
                reference_organism=args.reference_organism,
                expected_abundance_table=args.expected_abundance_table,
                control_samples_table=args.control_samples_table,
                value_transformation=args.value_transformation,
                sample_table=args.sample_table,
                sample_id_col=args.sample_id_col
                )


class NPAT:
    """NeuroProteomics Analysis Tool"""

    def __init__(self,
                 analysis_name:str,
                 env_file:str = "./.env",
                 gene_col:str = 'gene',
                 pathway_col:str = 'pathway',
                 pathway_gene_table:str = 'pathway_genes_no_duplicates',
                 reference_organism:str = 'Mus musculus',
                 expected_abundance_table:str = 'expected_abundance',
                 control_samples_table:str = 'control_samples',
                 value_transformation:str = 'log2',
                 sample_table:str = 'test_input',
                 sample_id_col:str = 'sample_id',
                 ):
        self.analysis_name = analysis_name
        self.connect_to_db(env_file=env_file)
        self.gene_col:str = gene_col
        self.pathway_col:str = pathway_col
        self.pathway_gene_table:str = pathway_gene_table
        self.reference_organism:str = reference_organism
        self.expected_abundance_table:str = expected_abundance_table
        self.control_samples_table:str = control_samples_table
        self.value_transformation:str = value_transformation
        self.sample_table:str = sample_table
        self.sample_id_col:str = sample_id_col

    def __str__(self):
        return self.analysis_name

    def connect_to_db(self, env_file:str) -> None:
        """Connect to DB with information stored in .env file"""
        connection_info = dotenv_values(env_file)
        DB_DRIVER = connection_info["DB_DRIVER"]
        DB_NAME = connection_info["DB_NAME"]
        DB_USER = connection_info["DB_USER"]
        DB_PASSWORD = connection_info["DB_PASSWORD"]
        DB_HOST = connection_info["DB_HOST"]
        DB_PORT = connection_info["DB_PORT"]

        url = URL.create(
            drivername=DB_DRIVER,
            username=DB_USER,
            password=DB_PASSWORD,
            host=DB_HOST,
            port=DB_PORT,
            database=DB_NAME
        )

        engine = create_engine(url)
        conn = engine.connect()

        self.engine = engine
        self.conn = conn

    def get_sample_ids(self) -> pd.DataFrame:
        """Return sample ids that are stored in sample table"""
        sql = f"""
        SELECT DISTINCT {self.sample_id_col} FROM {self.sample_table}
        """
        df = pd.read_sql(sql=sql, con=self.conn)
        return df[self.sample_id_col].sort_values(ascending=True, key=natsort_keygen())

    def get_samples(self, sample_ids: List[str]) -> pd.DataFrame:
        """Return a dataframe with the information for the given sample ids"""
        sql = f"""
        SELECT * FROM {self.sample_table}
        WHERE sample_id IN ({', '.join([f"'{x}'" for x in sample_ids])})
        """
        df = pd.read_sql(sql=sql, con=self.conn)
        return df

    def relative_expression_filter(self, gene_abundance_df: pd.DataFrame) -> pd.DataFrame:
        """Calculate relative expression of a gene by using Welch's t-test to see if sample and controls have the same mean. Return relative expression with type of expression(overexpressed/underexpressed)"""
        # TODO: Figure out if reading in all data then iterating through genes is faster than doing a sql query for each gene individually
        genes_list = gene_abundance_df.gene.unique()

        sql = f"""
        SELECT {self.gene_col}, VALUE FROM {self.control_samples_table}
        WHERE {self.gene_col} IN ({', '.join([f"'{x}'" for x in genes_list])})
        """

        control_samples_df = pd.read_sql(con=self.conn, sql=sql)

        relative_expression_dict = {'gene': [], 'expression': [], 'relative_expression': [], 'p_value': [],
                                    't_statistic': []}


        for gene in genes_list:
            control_values = control_samples_df.value[control_samples_df.gene == gene]
            test_values = gene_abundance_df.value[gene_abundance_df[self.gene_col] == gene]
            ttest_results = ttest_ind(test_values, control_values, equal_var=False)  # equal_var=False performs Welch’s t-test
            relative_expression = test_values.mean() - control_values.mean()  # log transformed values so need to use log arithmatic

            relative_expression_dict['gene'].append(gene)
            relative_expression_dict['relative_expression'].append(relative_expression)
            relative_expression_dict['p_value'].append(ttest_results.pvalue)
            relative_expression_dict['t_statistic'].append(ttest_results.statistic)

            if ttest_results.pvalue < 0.05:
                if ttest_results.statistic > 0:
                    relative_expression_dict['expression'].append('overexpressed')
                elif ttest_results.statistic < 0:
                    relative_expression_dict['expression'].append('suppressed')
            else:
                relative_expression_dict['expression'].append('expected')

        return pd.DataFrame.from_dict(relative_expression_dict)


    def pathway_gene_count(self, genes_list:List[str]) -> pd.DataFrame:
        """Returns pathways with the number of genes present from genes_list"""
        sql = f"""
        SELECT {self.pathway_col}, COUNT(*) AS sample_gene_count FROM {self.pathway_gene_table}
        WHERE {self.gene_col} IN ({', '.join([f"'{x}'" for x in genes_list])})
        GROUP BY {self.pathway_col}
        ORDER BY COUNT(*) DESC
        """

        pathway_gene_count_df = pd.read_sql(con=self.conn, sql=sql)
        return pathway_gene_count_df

    def expected_gene_count(self, pathway_gene_count_df:pd.DataFrame, num_genes_submitted:int) -> pd.DataFrame:
        """Calculates the number of genes that are expected to be present for each pathway containing genes from genes_list
        Steps:
        1) Count number of total genes present in pathway
        2) Divide total number of genes by number of genes present in reference organism to find reference gene ratio
        3) Find expected number of genes by multiplying reference gene ratio by the number of genes in genes_list
        """
        #TODO: Determine if size of gene list should be len of original gene list or filtered gene list after relative expression filter

        sql = f"""
        SELECT {self.pathway_col}, COUNT(*) AS total_gene_count_reference 
        FROM {self.pathway_gene_table}
        WHERE {self.pathway_col} IN ({', '.join([f"'{x}'" for x in pathway_gene_count_df.pathway])})
        GROUP BY pathway
        """

        expected_gene_count_df = pd.read_sql(sql=sql, con=self.conn)

        if self.reference_organism == 'Mus musculus':
            # Found from panther classification system which was launched from gene ontology site.
            # Number is found at start of results under reference list beside uniquely mapped ids.
            reference_genes_count = 21836

        expected_gene_count_df['reference_gene_ratio'] = expected_gene_count_df[
                                                             'total_gene_count_reference'] / reference_genes_count

        expected_gene_count_df['expected_count'] = expected_gene_count_df['reference_gene_ratio'] * num_genes_submitted

        return expected_gene_count_df

    def fold_change(self, pathway_gene_count_df:pd.DataFrame, expected_gene_count_df:pd.DataFrame, expression: str, num_genes_submitted: int) -> pd.DataFrame:
        """Calculate fold change by dividing sample_gene_count by expected_count (Panther method)
            Working on fold change by Berco method
        """
        fold_change_df = pathway_gene_count_df.merge(expected_gene_count_df, how='outer', on=self.pathway_col)

        fold_change_df['panther_fold_enrichment'] = fold_change_df['sample_gene_count'] / fold_change_df[
            'expected_count']
        if expression == 'overexpressed':
            fold_change_df['panther_fold_enrichment_p_value'] = fold_change_df.apply(lambda row: binomtest(k=row['sample_gene_count'], n=num_genes_submitted, p=row['reference_gene_ratio'], alternative='greater').pvalue, axis=1)  #TODO: Add p-value for binomial test from box 3 of Panther paper
        else:
            fold_change_df['panther_fold_enrichment_p_value'] = fold_change_df.apply(lambda row: binomtest(k=row['sample_gene_count'], n=num_genes_submitted, p=row['reference_gene_ratio'], alternative='less').pvalue, axis=1)

        # fold_change_df['npat_fold_enrichment'] =
        return fold_change_df

    def enrichment_analysis(self, gene_expression_df, expression='overexpressed'):
        """Run enrichment analysis on relative expression dataframe"""
        filtered_genes_list = gene_expression_df[gene_expression_df.expression == expression].gene.unique()
        num_genes_submitted = len(filtered_genes_list)
        pathway_gene_count_df = self.pathway_gene_count(genes_list=filtered_genes_list)
        expected_gene_count_df = self.expected_gene_count(pathway_gene_count_df=pathway_gene_count_df,
                                                          num_genes_submitted=num_genes_submitted)
        fold_change_df = self.fold_change(pathway_gene_count_df=pathway_gene_count_df,
                                          expected_gene_count_df=expected_gene_count_df,
                                          expression=expression,
                                          num_genes_submitted=num_genes_submitted)

        return fold_change_df


    def full_analysis(self, gene_abundance_df, expression='overexpressed'):
        """Run full analysis on gene abundance dataframe"""
        gene_expression_df = self.relative_expression_filter(gene_abundance_df=gene_abundance_df)
        fold_change_df = self.enrichment_analysis(gene_expression_df=gene_expression_df, expression=expression)

        return fold_change_df

def main() -> None:
    args = get_args()
    npat = NPAT(analysis_name=args.analysis_name,
                env_file=args.env_file,
                gene_col=args.gene_col,
                pathway_col=args.pathway_col,
                pathway_gene_table=args.pathway_gene_table,
                reference_organism=args.reference_organism,
                expected_abundance_table=args.expected_abundance_table,
                control_samples_table=args.control_samples_table,
                value_transformation=args.value_transformation,
                sample_table=args.sample_table,
                sample_id_col=args.sample_id_col)

if __name__ == '__main__':
    main()
        



import csv
from dotenv import dotenv_values
import pandas as pd
from sqlalchemy import create_engine
from sqlalchemy.engine import URL

class DBConnection:
    def __init__(self, env_file:str) -> None:
        self.connect_to_db(env_file=env_file)

    def connect_to_db(self, env_file:str) -> None:
        """Connect to DB with information stored in .env file"""
        connection_info = dotenv_values(env_file)
        DB_DRIVER = connection_info["DB_DRIVER"]
        DB_NAME = connection_info["DB_NAME"]
        DB_USER = connection_info["DB_USER"]
        DB_PASSWORD = connection_info["DB_PASSWORD"]
        DB_HOST = connection_info["DB_HOST"]
        DB_PORT = connection_info["DB_PORT"]

        url = URL.create(
            drivername=DB_DRIVER,
            username=DB_USER,
            password=DB_PASSWORD,
            host=DB_HOST,
            port=DB_PORT,
            database=DB_NAME
        )

        engine = create_engine(url)
        conn = engine.connect()

        self.engine = engine
        self.conn = conn

class NPATDB(DBConnection):
    def __init__(self, env_file:str) -> None:
        super().__init__(env_file)

    def format_proteome_data(self, proteome_tsv_path: str) -> pd.DataFrame:
        with open(proteome_tsv_path) as file:
            rd = csv.reader(file, delimiter="\t", quotechar='"')
            header = True
            proteome_data = []

            for row in rd:
                if header:
                    columns = row
                    header = False
                else:
                    gene_data = {}
                    for column, value in zip(columns, row):
                        gene_data[column] = value

                    gene = gene_data['Gene Names (primary)']
                    organism = gene_data['Organism']
                    reviewed = gene_data['Reviewed']
                    entry = gene_data['Entry']

                    if gene:
                        if gene_data.get('Gene Ontology (biological process)', False):
                            go_type = 'biological process'
                            for go in gene_data['Gene Ontology (biological process)'].split(';'):
                                tmp_dict = {
                                    'gene': gene,
                                    'pathway': go.strip(),
                                    'type': go_type,
                                    'organism': organism,
                                    'reviewed': reviewed,
                                    'entry': entry
                                }
                                proteome_data.append(tmp_dict)

                        if gene_data.get('Gene Ontology (molecular function)', False):
                            go_type = 'molecular function'
                            for go in gene_data['Gene Ontology (molecular function)'].split(';'):
                                tmp_dict = {
                                    'gene': gene,
                                    'pathway': go.strip(),
                                    'type': go_type,
                                    'organism': organism,
                                    'reviewed': reviewed,
                                    'entry': entry
                                }
                                proteome_data.append(tmp_dict)

                        if not gene_data.get('Gene Ontology (biological process)', False) and not gene_data.get(
                                'Gene Ontology (molecular function)', False):
                            go_type = None
                            tmp_dict = {
                                'gene': gene,
                                'pathway': go.strip(),
                                'type': go_type,
                                'organism': organism,
                                'reviewed': reviewed,
                                'entry': entry
                            }
                            proteome_data.append(tmp_dict)

        proteome_df = pd.DataFrame(proteome_data)
        proteome_df.sort_values('reviewed', ascending=True, inplace=True)
        proteome_df['id'] = proteome_df['gene'] + '_' + proteome_df['pathway']
        proteome_df.drop_duplicates(subset=['id'], keep='first', inplace=True)
        proteome_df.drop('id', inplace=True, axis=1)

        return proteome_df

    def create_genes_pathway_table(self, proteome_tsv_path: str) -> None:
        proteome_df = self.format_proteome_data(proteome_tsv_path)
        proteome_df.to_sql(name='pathway_genes', con=self.engine, if_exists='replace', index=False)

    def create_pathways_disease_table(self):
        pass

    def create_proteins_disease_table(self):
        pass

    def create_control_samples_table(self):
        pass

    def upload_samples_data(self):
        pass

    def creates_samples_table(self):
        pass

    def create_database(self):
        pass



# proteome_tsv_file = "./data/uniprotkb_proteome_UP000000589_2025_06_30.tsv"
# env_file = './local.env'
#
# db = NPATDB(env_file)
# db.create_genes_pathway_table(proteome_tsv_file)



from dotenv import dotenv_values
from neuroproteomics_analysis_tool.npat import NPAT
import numpy as np
import os
import pandas as pd
from textual import on
from textual.app import App, ComposeResult
from textual.containers import HorizontalScroll, VerticalScroll
from textual.css.query import NoMatches
from textual.reactive import reactive
from textual.widgets import TabbedContent, TabPane, Footer, Header,Label, Markdown, ListItem, ListView, Input, Static, Button, SelectionList, DataTable, Select


HOMEPAGE = """
# NeuroProteomics Analysis Tool (NPAT)

Welcome to the NeuroProteomics Analysis Tool (NPAT) brought to you by Berco Analytics and MoZeek Bio.
"""


def get_env_files(directory: str) -> list[str]:
    env_files = []
    for filename in os.listdir(directory):
        if filename.endswith(".env"):
            env_files.append(filename)
    return env_files


class NPATApp(App):
    """Need to add description"""
    CSS_PATH = ['tcss/analyses_analyses_layout.tcss',
                'tcss/analyses_enrichment_analysis_layout.tcss',
                'tcss/analyses_relative_expression_layout.tcss',
                'tcss/analyses_samples_layout.tcss',
                'tcss/database_connection_layout.tcss',
                'tcss/feedback_layout.tcss',
                'tcss/general_layout.tcss',
                'tcss/homepage_layout.tcss',]

    env_file_dir: reactive[str] = reactive(os.getcwd())
    env_file_path: reactive[str | None] = reactive(None)
    analyses_dict: reactive[dict[str, NPAT]] = reactive({})
    analysis_selection: reactive[str | None] = reactive(None)
    active_analysis: reactive[str | None] = reactive(None)
    sample_ids: reactive[dict[str, list]] = reactive({})
    relative_expressions: reactive[dict[str, pd.DataFrame]] = reactive({})
    enrichment_analyses: reactive[dict[str, pd.DataFrame]] = reactive({})


    BINDINGS = [
        ("q", "quit", "Quit"),
    ]

    def convert_dataframe_to_datatable(self, dataframe: pd.DataFrame, id:str, fixed_columns:int=0) -> None:
        self.query_one(f"#{id}").clear(columns=True)

        df_all_str = dataframe.copy(deep=True)
        numeric_column_names = df_all_str.select_dtypes(include=np.number).columns.tolist()
        for col in numeric_column_names:
            df_all_str[col] = df_all_str[col].astype(str)

        datatable = self.query_one(f'#{id}')
        datatable.cursor_type = "row"
        datatable.zebra_stripes = True
        datatable.fixed_columns = fixed_columns

        rows = list(df_all_str.itertuples(index=False, name=None))

        datatable.add_columns(*df_all_str.columns)
        datatable.add_rows(rows)

    def compose(self) -> ComposeResult:
        yield Header()
        #Footer to show key bindings
        yield Footer()

        with TabbedContent(initial='homepage_tab'):
            with TabPane(title='Homepage', id='homepage_tab'):
                yield Markdown(HOMEPAGE)
            with TabPane(title='Database Connection', id='database_tab'):
                with HorizontalScroll():
                    with VerticalScroll(id='env_file_pane'):
                        yield Static(id="env_dir_prompt")

                    with VerticalScroll(id='connection_info_pane', classes='hidden'):
                        yield Static(id='env_file_prompt')

                        with HorizontalScroll():
                            yield Static("DB_DRIVER: ", id='db_driver_label', classes='label')
                            yield Static(id='db_driver', classes='value')
                        with HorizontalScroll():
                            yield Static("DB_NAME: ", id='db_name_label', classes='label')
                            yield Static(id="db_name", classes='value')
                        with HorizontalScroll():
                            yield Static("DB_USER: ", id='db_user_label', classes='label')
                            yield Static(id='db_user', classes='value')
                        with HorizontalScroll():
                            yield Static("DB_PASSWORD: ", id='db_password_label', classes='label')
                            yield Static(id='db_password', classes='value')
                        with HorizontalScroll():
                            yield Static("DB_HOST: ", id='db_host_label', classes='label')
                            yield Static(id='db_host', classes='value')
                        with HorizontalScroll():
                            yield Static("DB_PORT: ", id='db_port_label', classes='label')
                            yield Static(id='db_port', classes='value')
                        with VerticalScroll(id='npat_connection_pane'):
                            yield Static(
                                '\nEnter an analysis name and click \"Connect to Database\" to create a new analysis')
                            yield Input(placeholder='Ex. Male Patient 35', id='analysis_name')
                            yield Button('Connect to Database', id='connect_db_button')
                            yield Static("\nActive Analyses")
                            yield ListView(id='active_analyses')

            with TabPane(title='Analyses', id='analyses_tab'):

                with TabbedContent(initial='analyses_homepage'):
                    with TabPane(title='Analyses', id='analyses_homepage'):
                        with HorizontalScroll():
                            with VerticalScroll():
                                yield Static('Set an active analysis.\n')
                                yield Static(id='active_analysis')
                                yield ListView(id='active_analyses_2')
                                yield Button('Set Active Analysis', id='set_active_analysis')
                            with VerticalScroll():
                                pass
                    with TabPane(title='Samples', id='samples_tab', classes='hidden'):
                        with HorizontalScroll(id='samples_top_pane'):
                            with VerticalScroll():
                                yield Static("Available Samples")
                                yield SelectionList(id='available_samples')
                            with VerticalScroll():
                                yield Static("Selected Samples")
                                yield ListView(id='selected_samples')
                                yield Button("Set Selected Samples", id='set_selected_samples')
                            with VerticalScroll():
                                yield Static("Active Samples")
                                yield ListView(id='active_samples')
                        with HorizontalScroll(id='samples_bottom_pane'):
                            yield Input(placeholder="Directory of gene abundance csv files to upload", id='gene_abundance_directory')
                            yield Button("Upload", id='upload_samples')

                    with TabPane(title='Relative Expression', id='relative_expression_tab', classes='hidden'):
                        yield Static('Relative Expression Description')
                        with HorizontalScroll():
                            with VerticalScroll(id='relative_expression_left_pane'):
                                yield Static('Active Samples')
                                yield ListView(id='active_samples_2')
                                yield Button("Calculate Relative Expression", id='calculate_relative_expression')
                            with VerticalScroll(id='relative_expression_rightq_pane'):
                                yield DataTable(id='relative_expression')
                                with HorizontalScroll():
                                    yield Input(placeholder='Path to export relative expression.', id='relative_expression_path')
                                    yield Button("Export", id='export_relative_expression_button')

                    with TabPane(title='Enrichment Analysis', id='enrichment_analysis_tab', classes='hidden'):
                        yield Static('Enrichment Analysis Description')
                        with VerticalScroll(id='enrichment_analysis_options'):
                            yield Static("Select Target Expression")
                            yield Select([('suppressed','suppressed'), ('overexpressed', 'overexpressed')],id='expression_selection', allow_blank=False)
                            yield Button('Run Enrichment Analysis', id='run_enrichment_analysis')
                        with VerticalScroll(id='enrichment_analysis_pane'):
                            yield DataTable(id='enrichment_analysis')
                            with HorizontalScroll():
                                yield Input(placeholder='Path to export enrichment analysis', id='enrichment_analysis_path')
                                yield Button("Export", id='export_enrichment_analysis_button')

            with TabPane(title='Feedback', id='feedback_tab'):
                yield Static("Thank you for using the NeuroProteomics Analysis Tool. We are looking for feedback to improve our tool. This page will be updated to allow submission of feedback.")
                # yield Select(prompt="Category (User Interface, Functionality)")
                # yield Select(prompt="SubCategory (Styling, Format/Layout, Errors/Performance, Relative Expression, Enrichment Analysis, Graph Analysis, New Analysis)")
                # yield Select(prompt="Location")
                # yield Input('Notes')
                # yield Button("Submit")

    def action_show_tab(self, tab: str) -> None:
        """Switch to a new tab"""
        self.get_child_by_type(TabbedContent).active = tab

    @on(message_type=Input.Submitted, selector="#env_dir_input")
    def env_dir_submitted(self):
        self.env_file_dir = self.query_one("#env_dir_input").value

    def watch_env_file_dir(self) -> None:
        container = self.query_one("#env_file_pane")

        if os.path.isdir(self.env_file_dir):
            env_files = get_env_files(self.env_file_dir)

            if len(env_files) == 0:
                self.query_one("#env_dir_prompt").update(f'No .env files were found in the directory: {self.env_file_dir}. Please provide a directory containing .env files with database connection information.')

                try:
                    self.query_one("#env_dir_input").value = ""

                except NoMatches:
                    container.mount(Input(placeholder='Please provide path to directory containing .env files', id='env_dir_input'))
            else:
                self.query_one("#env_dir_prompt").update(f'Please select the .env file you would like to use to connect to your database. Must contain the following variables: DB_DRIVER, DB_NAME, DB_USER, DB_PASSWORD, DB_HOST, DB_PORT\n\nDirectory: {self.env_file_dir}')
                try:
                    self.query_one("#env_dir_input").remove()
                except NoMatches:
                    pass
                container.mount(ListView(*[ListItem(Label(renderable=f'{file}', name=os.path.join(self.env_file_dir, file))) for file in env_files], id='env_files'))
        else:
            self.query_one("#env_dir_prompt").update(f"The following directory couldn't be found: {self.env_file_dir}. Please provide a directory containing .env files with database connection information.")
            self.query_one("#env_dir_input").value = ""

    @on(ListView.Selected, selector="#env_files")
    def env_file_submitted(self, selection: ListView.Selected):
        self.env_file_path = selection.item.query_one(Label).name

    def watch_env_file_path(self) -> None:
        if self.env_file_path:
            self.query_one("#connection_info_pane").remove_class("hidden")
            self.query_one("#env_file_prompt").update(f"You have selected the following .env file: {self.env_file_path}\n\nHover values to view.\n")
            connection_info = dotenv_values(self.env_file_path)
            self.query_one("#db_driver").update(connection_info.get("DB_DRIVER", "Value Missing"))
            self.query_one("#db_name").update(connection_info.get("DB_NAME", "Value Missing"))
            self.query_one("#db_user").update(connection_info.get("DB_USER", "Value Missing"))
            self.query_one("#db_password").update(connection_info.get("DB_PASSWORD", "Value Missing"))
            self.query_one("#db_host").update(connection_info.get("DB_HOST", "Value Missing"))
            self.query_one("#db_port").update(connection_info.get("DB_PORT", "Value Missing"))


    @on(Button.Pressed, selector="#connect_db_button")
    def connect_db_button_pressed(self) -> None:
        analysis_name = self.query_one("#analysis_name").value
        if analysis_name.strip():
            self.analyses_dict[analysis_name] = NPAT(analysis_name=analysis_name, env_file=self.env_file_path)
            self.mutate_reactive(NPATApp.analyses_dict)

        self.query_one("#analysis_name").value = ""

    def watch_analyses_dict(self) -> None:
        self.query_one("#active_analyses").clear()
        self.query_one("#active_analyses").extend([ListItem(Label(renderable=analysis, name=analysis)) for analysis in self.analyses_dict])

        self.query_one("#active_analyses_2").clear()
        self.query_one("#active_analyses_2").extend([ListItem(Label(renderable=analysis, name=analysis)) for analysis in self.analyses_dict])

    @on(ListView.Selected, selector="#active_analyses_2")
    def active_analysis_selected(self, selection: ListView.Selected):
        self.analysis_selection = selection.item.query_one(Label).name

    @on(Button.Pressed, selector="#set_active_analysis")
    def set_active_analysis(self):
        self.active_analysis = self.analysis_selection
        self.query_one("#samples_tab").remove_class("hidden")

    def watch_active_analysis(self) -> None:
        self.query_one("#active_analysis").update(f"Active Analysis: {self.active_analysis}")
        if self.active_analysis:
            sample_names = [(x,x) for x in self.analyses_dict[self.active_analysis].get_sample_ids()]

            self.query_one("#available_samples").clear_options()
            self.query_one("#available_samples").add_options(sample_names)

            self.query_one("#active_samples").clear()
            self.query_one("#active_samples_2").clear()
            self.query_one("#relative_expression").clear(columns=True)
            self.query_one("#enrichment_analysis").clear(columns=True)

            if self.sample_ids.get(self.active_analysis, None):
                self.query_one("#active_samples").extend([ListItem(Label(renderable=sample, name=sample)) for sample in self.sample_ids[self.active_analysis]])
                self.query_one("#active_samples_2").extend([ListItem(Label(renderable=sample, name=sample)) for sample in self.sample_ids[self.active_analysis]])

            if type(self.relative_expressions.get(self.active_analysis, None)) == pd.DataFrame:
                df = self.relative_expressions[self.active_analysis]
                self.convert_dataframe_to_datatable(dataframe=df, id='relative_expression')

            if type(self.enrichment_analyses.get(self.active_analysis, None)) == pd.DataFrame:
                df = self.enrichment_analyses[self.active_analysis]
                self.convert_dataframe_to_datatable(dataframe=df, id='enrichment_analysis', fixed_columns=1)





    @on(SelectionList.SelectedChanged, selector="#available_samples")
    def update_selected_samples(self) -> None:
        self.query_one("#selected_samples").clear()
        self.query_one("#selected_samples").extend([ListItem(Label(renderable=sample, name=sample)) for sample in self.query_one("#available_samples").selected])

    @on(Button.Pressed, selector="#set_selected_samples")
    def set_selected_samples(self) -> None:
        self.sample_ids[self.active_analysis] = self.query_one("#available_samples").selected

        if type(self.relative_expressions.get(self.active_analysis, None)) == pd.DataFrame:
            del self.relative_expressions[self.active_analysis]

        if type(self.enrichment_analyses.get(self.active_analysis, None)) == pd.DataFrame:
            del self.enrichment_analyses[self.active_analysis]

        self.query_one("#active_samples").clear()
        self.query_one("#active_samples").extend([ListItem(Label(renderable=sample, name=sample)) for sample in self.sample_ids[self.active_analysis]])

        self.query_one("#active_samples_2").clear()
        self.query_one("#active_samples_2").extend([ListItem(Label(renderable=sample, name=sample)) for sample in self.sample_ids[self.active_analysis]])

        self.query_one("#relative_expression").clear(columns=True)
        self.query_one("#enrichment_analysis").clear(columns=True)

        self.query_one("#available_samples").deselect_all()

        self.query_one("#relative_expression_tab").remove_class("hidden")



    @on(Button.Pressed, selector="#calculate_relative_expression")
    def calculate_relative_expression(self) -> None:
        df = self.analyses_dict[self.active_analysis].get_samples(self.sample_ids[self.active_analysis])
        df = self.analyses_dict[self.active_analysis].relative_expression_filter(df)
        self.relative_expressions[self.active_analysis] = df
        self.convert_dataframe_to_datatable(dataframe=df, id='relative_expression')

        self.query_one("#enrichment_analysis").clear(columns=True)

        if type(self.enrichment_analyses.get(self.active_analysis, None)) == pd.DataFrame:
            del self.enrichment_analyses[self.active_analysis]

        self.query_one("#enrichment_analysis_tab").remove_class("hidden")



    @on(Button.Pressed, selector="#run_enrichment_analysis")
    def run_enrichment_analysis(self) -> None:
        expression_selection = self.query_one('#expression_selection').selection
        if self.relative_expressions[self.active_analysis].expression.value_counts().get(expression_selection):
            df = self.analyses_dict[self.active_analysis].enrichment_analysis(gene_expression_df=self.relative_expressions[self.active_analysis],
                                                                              expression=expression_selection)
            self.enrichment_analyses[self.active_analysis] = df
            self.convert_dataframe_to_datatable(dataframe=df, id='enrichment_analysis', fixed_columns=1)

    @on(Button.Pressed, selector="#export_relative_expression_button")
    def export_relative_expression(self) -> None:
        filepath = self.query_one("#relative_expression_path").value
        dir = os.path.dirname(filepath)
        if dir != "":
            if not os.path.isdir(dir):
                os.makedirs(dir)
        self.relative_expressions[self.active_analysis].to_csv(filepath, index=False)
        self.query_one("#relative_expression_path").value = ""

    @on(Button.Pressed, selector="#export_enrichment_analysis_button")
    def export_enrichment_analysis(self) -> None:
        filepath = self.query_one("#enrichment_analysis_path").value
        dir = os.path.dirname(filepath)
        if dir != "":
            if not os.path.isdir(dir):
                os.makedirs(dir)
        self.enrichment_analyses[self.active_analysis].to_csv(filepath, index=False)
        self.query_one("#enrichment_analysis_path").value = ""

    @on(Button.Pressed, selector="#upload_samples")
    def upload_samples(self) -> None:
        directory = self.query_one("#gene_abundance_directory").value
        if os.path.exists(directory):
            sample_ids = self.analyses_dict[self.active_analysis].get_sample_ids()
            files = os.listdir(directory)
            files = [os.path.join(directory, file) for file in files]

            table_name = self.analyses_dict[self.active_analysis].sample_table
            for file in files:
                if file.endswith('.csv'):
                    dataframe = pd.read_csv(file)
                    dataframe = dataframe[~dataframe.sample_id.isin(sample_ids)]
                    if dataframe.shape[0] > 0:
                        pd.read_csv(file).to_sql(table_name, con=self.analyses_dict[self.active_analysis].conn, if_exists='append', index=False)

        self.query_one("#gene_abundance_directory").value = ''

        sample_names = [(x, x) for x in self.analyses_dict[self.active_analysis].get_sample_ids()]

        self.query_one("#available_samples").clear_options()
        self.query_one("#available_samples").add_options(sample_names)


    def on_mount(self) -> None:
        self.title = "NeuroProteomics Analysis Tool"
        self.sub_title = "Berco Analytics/MoZeek Bio"

if __name__ == '__main__':
    NPATApp().run()